@@ -1,16 +1,19 @@
-from aenum import Enum, IntEnum, Flag, UniqueEnum, AutoEnum, NamedTuple, TupleSize, AutoValue, AutoNumber, NoAlias, Unique, MultiValue
-from aenum import AutoNumberEnum, OrderedEnum, unique, skip, extend_enum
+from aenum import EnumMeta, Enum, IntEnum, Flag, UniqueEnum, AutoEnum
+from aenum import NamedTuple, TupleSize, MagicValue, AddValue, NoAlias, Unique, MultiValue
+from aenum import AutoNumberEnum,MultiValueEnum, OrderedEnum, unique, skip, extend_enum
 
 from collections import OrderedDict
 from datetime import timedelta
 from pickle import dumps, loads, PicklingError, HIGHEST_PROTOCOL
 from unittest import TestCase, main
 
+import os
 import tempfile
 import textwrap
 import sys
 pyver = float('%s.%s' % sys.version_info[:2])
 
+
 class TestEnumV3(TestCase):
 
     def setUp(self):
@@ -78,58 +81,44 @@ class TestEnumV3(TestCase):
         self.assertEqual(Color.BLUE.rgb, (0, 1, 0))
         self.assertEqual(Color.GREEN.rgb, (0, 0, 1))
 
-    # def test_auto_turns_off(self):
-    #     with self.assertRaises(NameError):
-    #         class Color(Enum, settings=AutoValue):
-    #             red
-    #             green
-    #             blue
-    #             def hello(self):
-    #                 print('Hello!  My serial is %s.' % self.value)
-    #             rose
-    #     with self.assertRaises(NameError):
-    #         class Color(Enum, settings=AutoValue):
-    #             red
-    #             green
-    #             blue
-    #             def __init__(self, *args):
-    #                 pass
-    #             rose
+    def test_auto_turns_off(self):
+        with self.assertRaises(NameError):
+            class Color(Enum, settings=MagicValue):
+                red
+                green
+                blue
+                def hello(self):
+                    print('Hello!  My serial is %s.' % self.value)
+                rose
+        with self.assertRaises(NameError):
+            class Color(Enum, settings=MagicValue):
+                red
+                green
+                blue
+                def __init__(self, *args):
+                    pass
+                rose
 
-    # def test_magic(self):
-    #     class Color(Enum, settings=AutoValue):
-    #         red, green, blue
-    #     self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
-    #     self.assertEqual(Color.red.value, 1)
+    def test_magic(self):
+        class Color(Enum, settings=MagicValue):
+            red, green, blue
+        self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
+        self.assertEqual(Color.red.value, 1)
 
-    # def test_ignore_not_overridden(self):
-    #     with self.assertRaisesRegex(TypeError, 'object is not callable'):
-    #         class Color(Flag):
-    #             _ignore_ = 'irrelevent'
-    #             _settings_ = AutoValue
-    #             @property
-    #             def shade(self):
-    #                 print('I am light', self.name.lower())
+    def test_ignore_not_overridden(self):
+        with self.assertRaisesRegex(TypeError, 'object is not callable'):
+            class Color(Flag):
+                _ignore_ = 'irrelevent'
+                _settings_ = MagicValue
+                @property
+                def shade(self):
+                    print('I am light', self.name.lower())
 
-    # def test_magic_start(self):
-    #     class Color(Enum, start=0):
-    #         red, green, blue
-    #     self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
-    #     self.assertEqual(Color.red.value, 0)
-
-    # def test_magic_on_and_off(self):
-    #     class Color(Enum):
-    #         _auto_on_
-    #         red
-    #         green
-    #         _auto_off_
-    #         @property
-    #         def cap_name(self) -> str:
-    #             return self.name.title()
-    #         _auto_on_
-    #         blue
-    #     self.assertEqual(len(Color), 3)
-    #     self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
+    def test_magic_start(self):
+        class Color(Enum, settings=MagicValue, start=0):
+            red, green, blue
+        self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
+        self.assertEqual(Color.red.value, 0)
 
     def test_dir_on_class(self):
         Season = self.Season
@@ -273,8 +262,15 @@ class TestEnumV3(TestCase):
                 green = 2, 'green'
                 blue = 3, 'blue', 'red'
 
-    def test_auto_and_init(self):
-        class Field(IntEnum, settings=AutoValue, init='value __doc__'):
+    def test_multivalue_and_auto(self):
+        with self.assertRaisesRegex(TypeError, r'MultiValue and MagicValue are mutually exclusive'):
+            class Color(Enum, settings=(MultiValue, MagicValue)):
+                red
+                green = 3, 'green'
+                blue
+
+    def test_autonumber_and_init(self):
+        class Field(IntEnum, settings=AddValue, init='__doc__'):
             TYPE = "Char, Date, Logical, etc."
             START = "Field offset in record"
         self.assertEqual(Field.TYPE, 1)
@@ -283,8 +279,15 @@ class TestEnumV3(TestCase):
         self.assertEqual(Field.START.__doc__, 'Field offset in record')
         self.assertFalse(hasattr(Field, '_order_'))
 
-    def test_auto_and_start(self):
-        class Field(IntEnum, init='__doc__', start=0):
+    def test_autovalue_and_init(self):
+        class Field(IntEnum, init='value __doc__'):
+            TYPE = "Char, Date, Logical, etc."
+            START = "Field offset in record"
+        self.assertEqual(Field.TYPE, 1)
+        self.assertEqual(Field.START.__doc__, 'Field offset in record')
+
+    def test_autonumber_and_start(self):
+        class Field(IntEnum, init='__doc__', settings=AddValue, start=0):
             TYPE = "Char, Date, Logical, etc."
             START = "Field offset in record"
         self.assertEqual(Field.TYPE, 0)
@@ -292,8 +295,8 @@ class TestEnumV3(TestCase):
         self.assertEqual(Field.TYPE.__doc__, 'Char, Date, Logical, etc.')
         self.assertEqual(Field.START.__doc__, 'Field offset in record')
 
-    def test_auto_and_init_and_some_values(self):
-        class Field(IntEnum, init='value __doc__', settings=AutoValue):
+    def test_autonumber_and_init_and_some_values(self):
+        class Field(IntEnum, init='value __doc__'):
             TYPE = "Char, Date, Logical, etc."
             START = "Field offset in record"
             BLAH = 5, "test blah"
@@ -307,44 +310,109 @@ class TestEnumV3(TestCase):
         self.assertEqual(Field.BLAH.__doc__, 'test blah')
         self.assertEqual(Field.BELCH.__doc__, 'test belch')
 
-    # def test_autoenum(self):
-    #     class Color(AutoEnum):
-    #         red
-    #         green
-    #         blue
-    #     self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
-    #     self.assertEqual([m.value for m in Color], [1, 2, 3])
-    #     self.assertEqual([m.name for m in Color], ['red', 'green', 'blue'])
+    def test_autonumber_with_irregular_values(self):
+        class Point(AutoNumberEnum, init='x y'):
+            first = 7, 9
+            second = 11, 13
+        self.assertEqual(Point.first.value, 1)
+        self.assertEqual(Point.first.x, 7)
+        self.assertEqual(Point.first.y, 9)
+        self.assertEqual(Point.second.value, 2)
+        self.assertEqual(Point.second.x, 11)
+        self.assertEqual(Point.second.y, 13)
+        with self.assertRaisesRegex(TypeError, '.*number of fields provided do not match init ...x., .y.. != .3, 11, 13..'):
+            class Point(AutoNumberEnum, init='x y'):
+                first = 7, 9
+                second = 3, 11, 13
+        class Color(AutoNumberEnum, init='__doc__'):
+            # interactions between AutoNumberEnum and _generate_next_value_ may not be pretty
+            red = ()
+            green = 'red'
+            blue = ()
+        self.assertTrue(Color.red.__doc__, 1)
+        self.assertEqual(Color.green.__doc__, 'red')
+        self.assertTrue(Color.blue.__doc__, 2)
 
-    # def test_autoenum_with_str(self):
-    #     class Color(AutoEnum):
-    #         def _generate_next_value_(name, start, count, last_values):
-    #             return name
-    #         red
-    #         green
-    #         blue
-    #     self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
-    #     self.assertEqual([m.value for m in Color], ['red', 'green', 'blue'])
-    #     self.assertEqual([m.name for m in Color], ['red', 'green', 'blue'])
+    def test_autonumber_and_property(self):
+        with self.assertRaises(TypeError):
+            class Color(AutoEnum):
+                _ignore_ = ()
+                red = ()
+                green = ()
+                blue = ()
+                @property
+                def cap_name(self) -> str:
+                    return self.name.title()
 
-    # def test_autoenum_and_default_ignore(self):
-    #     class Color(AutoEnum):
-    #         red
-    #         green
-    #         blue
-    #         @property
-    #         def cap_name(self):
-    #             return self.name.title()
-    #     self.assertEqual(Color.blue.cap_name, 'Blue')
+    def test_autoenum(self):
+        class Color(AutoEnum):
+            red
+            green
+            blue
+        self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
+        self.assertEqual([m.value for m in Color], [1, 2, 3])
+        self.assertEqual([m.name for m in Color], ['red', 'green', 'blue'])
 
-    # def test_combine_new_settings_with_old_settings(self):
-    #     class Auto(Enum, settings=Unique):
-    #         pass
-    #     with self.assertRaises(ValueError):
-    #         class AutoUnique(Auto, settings=AutoValue):
-    #             BLAH
-    #             BLUH
-    #             ICK = 1
+    def test_autoenum_with_str(self):
+        class Color(AutoEnum):
+            def _generate_next_value_(name, start, count, last_values):
+                return name
+            red
+            green
+            blue
+        self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
+        self.assertEqual([m.value for m in Color], ['red', 'green', 'blue'])
+        self.assertEqual([m.name for m in Color], ['red', 'green', 'blue'])
+
+    def test_autoenum_and_default_ignore(self):
+        class Color(AutoEnum):
+            red
+            green
+            blue
+            @property
+            def cap_name(self):
+                return self.name.title()
+        self.assertEqual(Color.blue.cap_name, 'Blue')
+
+    def test_autonumber_and_overridden_ignore(self):
+        with self.assertRaises(TypeError):
+            class Color(AutoEnum):
+                _ignore_ = 'staticmethod'
+                red
+                green
+                blue
+                @property
+                def cap_name(self) -> str:
+                    return self.name.title()
+
+    def test_autonumber_and_multiple_assignment(self):
+        class Color(AutoEnum):
+            _ignore_ = 'property'
+            red
+            green
+            blue = cyan
+            @property
+            def cap_name(self) -> str:
+                return self.name.title()
+        self.assertEqual(Color.blue.cap_name, 'Cyan')
+
+    def test_multivalue_and_autonumber_inherited(self):
+        class Measurement(int, Enum, settings=(MultiValue, AddValue), start=0):
+            one = "20110721"
+            two = "20120911"
+            three = "20110518"
+        M = Measurement
+        self.assertEqual(M.one, 0)
+        self.assertTrue(M.one is M(0) is M('20110721'))
+
+    def test_combine_new_settings_with_old_settings(self):
+        class Auto(Enum, settings=Unique):
+            pass
+        with self.assertRaises(ValueError):
+            class AutoUnique(Auto, settings=MagicValue):
+                BLAH
+                BLUH
+                ICK = 1
 
     def test_timedelta(self):
         class Period(timedelta, Enum):
@@ -391,17 +459,17 @@ class TestEnumV3(TestCase):
         self.assertEqual(len(Color), 4)
         self.assertEqual(Color.red.value, 1)
 
-    # def test_extend_enum_generate(self):
-    #     class Foo(AutoEnum):
-    #         def _generate_next_value_(name, start, count, values, *args, **kwds):
-    #             return name
-    #         a
-    #         b
-    #     #
-    #     extend_enum(Foo, 'c')
-    #     self.assertEqual(Foo.a.value, 'a')
-    #     self.assertEqual(Foo.b.value, 'b')
-    #     self.assertEqual(Foo.c.value, 'c')
+    def test_extend_enum_generate(self):
+        class Foo(AutoEnum):
+            def _generate_next_value_(name, start, count, values, *args, **kwds):
+                return name
+            a
+            b
+        #
+        extend_enum(Foo, 'c')
+        self.assertEqual(Foo.a.value, 'a')
+        self.assertEqual(Foo.b.value, 'b')
+        self.assertEqual(Foo.c.value, 'c')
 
     def test_extend_enum_unique_with_duplicate(self):
         with self.assertRaises(ValueError):
@@ -453,68 +521,68 @@ class TestEnumV3(TestCase):
             triple = 3
             value = 4
 
-    # def test_auto_number(self):
-    #     class Color(Enum, settings=AutoValue):
-    #         red
-    #         blue
-    #         green
-    #
-    #     self.assertEqual(list(Color), [Color.red, Color.blue, Color.green])
-    #     self.assertEqual(Color.red.value, 1)
-    #     self.assertEqual(Color.blue.value, 2)
-    #     self.assertEqual(Color.green.value, 3)
+    def test_auto_number(self):
+        class Color(Enum, settings=MagicValue):
+            red
+            blue
+            green
 
-    # def test_auto_name(self):
-    #     class Color(Enum, settings=AutoValue):
-    #         def _generate_next_value_(name, start, count, last):
-    #             return name
-    #         red
-    #         blue
-    #         green
-    #
-    #     self.assertEqual(list(Color), [Color.red, Color.blue, Color.green])
-    #     self.assertEqual(Color.red.value, 'red')
-    #     self.assertEqual(Color.blue.value, 'blue')
-    #     self.assertEqual(Color.green.value, 'green')
+        self.assertEqual(list(Color), [Color.red, Color.blue, Color.green])
+        self.assertEqual(Color.red.value, 1)
+        self.assertEqual(Color.blue.value, 2)
+        self.assertEqual(Color.green.value, 3)
 
-    # def test_auto_name_inherit(self):
-    #     class AutoNameEnum(Enum):
-    #         def _generate_next_value_(name, start, count, last):
-    #             return name
-    #     class Color(AutoNameEnum, settings=AutoValue):
-    #         red
-    #         blue
-    #         green
-    #
-    #     self.assertEqual(list(Color), [Color.red, Color.blue, Color.green])
-    #     self.assertEqual(Color.red.value, 'red')
-    #     self.assertEqual(Color.blue.value, 'blue')
-    #     self.assertEqual(Color.green.value, 'green')
+    def test_auto_name(self):
+        class Color(Enum, settings=MagicValue):
+            def _generate_next_value_(name, start, count, last):
+                return name
+            red
+            blue
+            green
 
-    # def test_auto_garbage(self):
-    #     class Color(Enum):
-    #         _settings_ = AutoValue
-    #         red = 'red'
-    #         blue
-    #     self.assertEqual(Color.blue.value, 1)
+        self.assertEqual(list(Color), [Color.red, Color.blue, Color.green])
+        self.assertEqual(Color.red.value, 'red')
+        self.assertEqual(Color.blue.value, 'blue')
+        self.assertEqual(Color.green.value, 'green')
 
-    # def test_auto_garbage_corrected(self):
-    #     class Color(Enum, settings=AutoValue):
-    #         red = 'red'
-    #         blue = 2
-    #         green
-    #
-    #     self.assertEqual(list(Color), [Color.red, Color.blue, Color.green])
-    #     self.assertEqual(Color.red.value, 'red')
-    #     self.assertEqual(Color.blue.value, 2)
-    #     self.assertEqual(Color.green.value, 3)
+    def test_auto_name_inherit(self):
+        class AutoNameEnum(Enum):
+            def _generate_next_value_(name, start, count, last):
+                return name
+        class Color(AutoNameEnum, settings=MagicValue):
+            red
+            blue
+            green
 
-    # def test_duplicate_auto(self):
-    #     class Dupes(Enum, settings=AutoValue):
-    #         first = primero
-    #         second
-    #         third
-    #     self.assertEqual([Dupes.first, Dupes.second, Dupes.third], list(Dupes))
+        self.assertEqual(list(Color), [Color.red, Color.blue, Color.green])
+        self.assertEqual(Color.red.value, 'red')
+        self.assertEqual(Color.blue.value, 'blue')
+        self.assertEqual(Color.green.value, 'green')
+
+    def test_auto_garbage(self):
+        class Color(Enum):
+            _settings_ = MagicValue
+            red = 'red'
+            blue
+        self.assertEqual(Color.blue.value, 1)
+
+    def test_auto_garbage_corrected(self):
+        class Color(Enum, settings=MagicValue):
+            red = 'red'
+            blue = 2
+            green
+
+        self.assertEqual(list(Color), [Color.red, Color.blue, Color.green])
+        self.assertEqual(Color.red.value, 'red')
+        self.assertEqual(Color.blue.value, 2)
+        self.assertEqual(Color.green.value, 3)
+
+    def test_duplicate_auto(self):
+        class Dupes(Enum, settings=MagicValue):
+            first = primero
+            second
+            third
+        self.assertEqual([Dupes.first, Dupes.second, Dupes.third], list(Dupes))
 
     def test_order_as_function(self):
         # first with _init_
@@ -1075,7 +1143,7 @@ class TestStackoverflowAnswersV3(TestCase):
 
     def test_create_C_like_Enum(self):
         # https://stackoverflow.com/a/35965438/208880
-        class Id(Enum, start=0):
+        class Id(Enum, settings=MagicValue, start=0):
             #
             NONE  # 0x0
             HEARTBEAT  # 0x1
@@ -1110,10 +1178,128 @@ class TestStackoverflowAnswersV3(TestCase):
         self.assertEqual(Id.NONE.value, 0)
         self.assertEqual(Id.FLUID_TRANSFER_ERROR_MSG.value, 4)
         self.assertEqual(Id.START_SENDING_PICTURES.value, 0x010000)
-        self.assertEqual(Id.STOP_RECORDING_VIDEO_REQ.Value, 0x010003)
+        self.assertEqual(Id.STOP_RECORDING_VIDEO_REQ.value, 0x010003)
         self.assertEqual(Id.START_CAL.value, 0x020001)
         self.assertEqual(Id.LAST_ID.value, 0x30005)
 
+
+    if pyver >= 3.5:
+        def test_c_header_scanner(self):
+            # https://stackoverflow.com/questions/58732872/208880
+            with open(os.path.join(tempdir, 'c_plus_plus.h'), 'w') as fh:
+                fh.write("""
+                        stuff before
+                        enum hello {
+                            Zero,
+                            One,
+                            Two,
+                            Three,
+                            Five=5,
+                            Six,
+                            Ten=10
+                            };
+                        in the middle
+                        enum blah
+                            {
+                            alpha,
+                            beta,
+                            gamma = 10 ,
+                            zeta = 50
+                            };
+                        at the end
+                        """)
+            from pyparsing import Group, Optional, Suppress, Word, ZeroOrMore
+            from pyparsing import alphas, alphanums, nums
+            #
+            CPPEnum = None
+            class CPPEnumType(EnumMeta):
+                #
+                @classmethod
+                def __prepare__(metacls, clsname, bases, **kwds):
+                    # return a standard dictionary for the initial processing
+                    return {}
+                #
+                def __init__(clsname, *args , **kwds):
+                    super(CPPEnumType, clsname).__init__(*args)
+                #
+                def __new__(metacls, clsname, bases, clsdict, **kwds):
+                    if CPPEnum is None:
+                        # first time through, ignore the rest
+                        enum_dict = super(CPPEnumType, metacls).__prepare__(clsname, bases, **kwds)
+                        enum_dict.update(clsdict)
+                        return super(CPPEnumType, metacls).__new__(metacls, clsname, bases, enum_dict, **kwds)
+                    members = []
+                    #
+                    # remove _file and _name using `pop()` as they will cause problems in EnumMeta
+                    try:
+                        file = clsdict.pop('_file')
+                    except KeyError:
+                        raise TypeError('_file not specified')
+                    cpp_enum_name = clsdict.pop('_name', clsname.lower())
+                    with open(file) as fh:
+                        file_contents = fh.read()
+                    #
+                    # syntax we don't want to see in the final parse tree
+                    LBRACE, RBRACE, EQ, COMMA = map(Suppress, "{}=,")
+                    _enum = Suppress("enum")
+                    identifier = Word(alphas, alphanums + "_")
+                    integer = Word(nums)
+                    enumValue = Group(identifier("name") + Optional(EQ + integer("value")))
+                    enumList = Group(enumValue + ZeroOrMore(COMMA + enumValue))
+                    enum = _enum + identifier("enum") + LBRACE + enumList("names") + RBRACE
+                    #
+                    # find the cpp_enum_name ignoring other syntax and other enums
+                    for item, start, stop in enum.scanString(file_contents):
+                        if item.enum != cpp_enum_name:
+                            continue
+                        id = 0
+                        for entry in item.names:
+                            if entry.value != "":
+                                id = int(entry.value)
+                            members.append((entry.name.upper(), id))
+                            id += 1
+                    #
+                    # get the real EnumDict
+                    enum_dict = super(CPPEnumType, metacls).__prepare__(clsname, bases, **kwds)
+                    # transfer the original dict content, names starting with '_' first
+                    items = list(clsdict.items())
+                    items.sort(key=lambda p: (0 if p[0][0] == '_' else 1, p))
+                    for name, value in items:
+                        enum_dict[name] = value
+                    # add the members
+                    for name, value in members:
+                        enum_dict[name] = value
+                    return super(CPPEnumType, metacls).__new__(metacls, clsname, bases, enum_dict, **kwds)
+            #
+            class CPPEnum(IntEnum, metaclass=CPPEnumType):
+                pass
+            #
+            class Hello(CPPEnum):
+                _file = os.path.join(tempdir, 'c_plus_plus.h')
+            #
+            class Blah(CPPEnum):
+                _file = os.path.join(tempdir, 'c_plus_plus.h')
+                _name = 'blah'
+            #
+            self.assertEqual(
+                    list(Hello),
+                    [Hello.ZERO, Hello.ONE, Hello.TWO, Hello.THREE, Hello.FIVE, Hello.SIX, Hello.TEN],
+                    )
+            self.assertEqual(Hello.ZERO.value, 0)
+            self.assertEqual(Hello.THREE.value, 3)
+            self.assertEqual(Hello.SIX.value, 6)
+            self.assertEqual(Hello.TEN.value, 10)
+            #
+            self.assertEqual(
+                    list(Blah),
+                    [Blah.ALPHA, Blah.BETA, Blah.GAMMA, Blah.ZETA],
+                    )
+            self.assertEqual(Blah.ALPHA.value, 0)
+            self.assertEqual(Blah.BETA.value, 1)
+            self.assertEqual(Blah.GAMMA.value, 10)
+            self.assertEqual(Blah.ZETA.value, 50)
+
+
 if __name__ == '__main__':
     tempdir = tempfile.mkdtemp()
     try:
